require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

SP_ALLOCATION_SCHEDULER_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

SAVINGS_PLANS_FMTRS =
    YAML.load_file(File.join(__dir__, '../helpers/sp_api_formatters.yml'))

# module for sp-allocation-scheduler
module ChtApitestSpAllocationScheduler
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module SpAllocationSchedulerHelpers
    # insertion point for shortcuts
    def create_sp_recommendation(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('create_sp_recommendation')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new))
      data = format_request(sym, args)
      variables = { customerInfo: { accountId: args[:accountId], customerId: args[:customerId], organizationId: args[:organizationId] }, input: { accountIds: args[:accountIds], evaluationPeriod: args[:evaluationPeriod], mode: args[:mode], paymentOption: args[:paymentOption], purchaseDate: args[:purchaseDate], quoteId: args[:quoteId], term: args[:term], updateEvent: { endpoint: args[:endpoint], query: args[:query], variables: args[:variables] } } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def create_default_sp_recommendation(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('create_default_sp_recommendation')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"customOptions"=>{"force"=>args[:force], "queue"=>args[:queue]}, "superartifactInfo"=>{"accountId"=>args[:accountId], "customerId"=>args[:customerId], "month"=>args[:month], "name"=>args[:name]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    def node(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('node')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
          auth_service: LocalAuthService.new))
      data = format_request(sym, args)
      variables =
        graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def sp_recommendations(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('sp_recommendations')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new))
      data = format_request(sym, args)
      variables = { customerInfo: { accountId: args[:accountId], customerId: args[:customerId], organizationId: args[:organizationId] }, quoteId: args[:quoteId] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def default_savings_plan_recommendations(args = {})
      args[:service] = SavingsPlansService.new
      sym = first_no_cap(camelize('default_savings_plan_recommendations')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: lauth_service))
      data = format_request(sym, args)
      variables = {  }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def artifact_job_statuses(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('artifact_job_statuses')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"artifactType"=>args[:artifactType], "customerInfo"=>{"accountId"=>args[:accountId], "customerId"=>args[:customerId], "organizationId"=>args[:organizationId]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    def refresh_artifact(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('refresh_artifact')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"artifactType"=>args[:artifactType], "customOptions"=>{"force"=>args[:force], "queue"=>args[:queue]}, "customerInfo"=>{"accountId"=>args[:accountId], "customerId"=>args[:customerId], "organizationId"=>args[:organizationId]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    # end of shortcut insertion
  end

  # class for your service
  class SpAllocationSchedulerService < ChtNgService
    include RSpec::Matchers

    SP_ALLOCATION_SCHEDULER_DEFAULTS = {
      name: 'SpAllocationScheduler',
      default_namespace: 'sp-allocation-scheduler',
      service_name: 'sp-allocation-scheduler',
      pr_parent: 'sp-allocation-scheduler',
      formatters: SP_ALLOCATION_SCHEDULER_FMTRS['formatters'],
      checkers: SP_ALLOCATION_SCHEDULER_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(SP_ALLOCATION_SCHEDULER_DEFAULTS))
    end
  end

  class SavingsPlansService < ChtNgService
    include RSpec::Matchers

    SAVINGS_PLANS_DEFAULTS = {
        name: 'SavingsPlans',
        default_namespace: 'savings-plans',
        service_name: 'savings-plans',
        pr_parent: 'savings-plans',
        formatters: SAVINGS_PLANS_FMTRS['formatters'],
        checkers: SAVINGS_PLANS_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(SAVINGS_PLANS_DEFAULTS))
    end
  end

  # class for api-proxy support of sp-allocation-scheduler service
  class ApiProxySpAllocationScheduler < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxySpAllocationScheduler',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: SP_ALLOCATION_SCHEDULER_FMTRS['formatters'],
      checkers: SP_ALLOCATION_SCHEDULER_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
    end
  end
end
