require 'spec_helper'

module SpAllocationSchedulerTest
  describe 'SpAllocationScheduler artifactJobStatuses' do
    include_context 'request metadata'

    customer_info = CUSTOMER_INFO

    it 'artifactJobStatuses request: Completed DOT jobs' do
      wrap_request do
        variables = {
          artifactType: 'DOT',
          customerInfo: customer_info,
          filterRules: [
            {field: "status",
             filterType: "EQ",
             filterValues: ["COMPLETED"]}
          ],
          sortRules: [],
        }
        graphql url(service: service),
          format_request(
            :artifactJobStatuses,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:artifactJobStatuses, service: service)
      end
      job_array = json_body[:data][:artifactJobStatuses][:edges]
      expect(job_array.length).to be > 0
      last_created_date = Time.parse(job_array[0][:node][:createdAt])
      different_dates_present = false
#       job_array.each {|job|
#         expect(job[:node][:artifactType]).to eq('DOT')
#         expect(job[:node][:status]).to eq('COMPLETED')
#         expect(job[:node][:failedReason]).to eq(nil)
#         expect(Time.parse(job[:node][:createdAt])).to be <= Time.parse(job[:node][:finishedAt])
#         expect(job[:node][:accountId]).to eq((TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id])
#         expect(job[:node][:customerId]).to eq((TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:customer_id] : TEST_CUSTOMER[:customer_id])
#
#         expect(Time.parse(job[:node][:createdAt])).to be >= last_created_date
#         if Time.parse(job[:node][:createdAt]) > last_created_date
#           different_dates_present = true
#         end
#         last_created_date = Time.parse(job[:node][:createdAt])
#       }
#       if job_array.length > 10
#         expect(different_dates_present).to eq(true),
#           "All jobs in artifactJobStatuses job array have same createdAt date : #{job_array}"
#       end
    end

    # the following tests don't verify a minimum number of nodes,
    # so won't fail if no nodes with a the status under test are available
    # meant to predominantly verify against schema issues/regressions

    it 'artifactJobStatuses request: CREATED state jobs' do
      wrap_request do
        variables = {
          artifactType: 'DOT',
          customerInfo: customer_info,
          filterRules: [
            {field: "status",
             filterType: "EQ",
             filterValues: ["CREATED"]}
          ],
          sortRules: [],
        }
        graphql url(service: service),
          format_request(
            :artifactJobStatuses,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:artifactJobStatuses, service: service)
      end
      json_body[:data][:artifactJobStatuses][:edges].each {|job|
        expect(job[:node][:status]).to eq('CREATED')
      }
    end

    it 'artifactJobStatuses request: GENERATING state jobs' do
      wrap_request do
        variables = {
          artifactType: 'DOT',
          customerInfo: customer_info,
          filterRules: [
            {field: "status",
             filterType: "EQ",
             filterValues: ["GENERATING"]}
          ],
          sortRules: [],
        }
        graphql url(service: service),
          format_request(
            :artifactJobStatuses,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:artifactJobStatuses, service: service)
      end
      json_body[:data][:artifactJobStatuses][:edges].each {|job|
        expect(job[:node][:status]).to eq('GENERATING')
      }
    end

    it 'artifactJobStatuses request: FLEXREPORT_SETUP state jobs' do
      wrap_request do
        variables = {
          artifactType: 'DOT',
          customerInfo: customer_info,
          filterRules: [
            {field: "status",
             filterType: "EQ",
             filterValues: ["FLEXREPORT_SETUP"]}
          ],
          sortRules: [],
        }
        graphql url(service: service),
          format_request(
            :artifactJobStatuses,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:artifactJobStatuses, service: service)
      end
      json_body[:data][:artifactJobStatuses][:edges].each {|job|
        expect(job[:node][:status]).to eq('FLEXREPORT_SETUP')
      }
    end

    it 'artifactJobStatuses request: Failed jobs' do
      wrap_request do
        variables = {
          artifactType: 'DOT',
          customerInfo: customer_info,
          filterRules: [
            {field: "status",
             filterType: "EQ",
             filterValues: ["FAILED"]}
          ],
          sortRules: [],
        }
        graphql url(service: service),
          format_request(
            :artifactJobStatuses,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:artifactJobStatuses, service: service)
      end
      json_body[:data][:artifactJobStatuses][:edges].each {|job|
        expect(job[:node][:status]).to eq('FAILED')
      }
    end
  end
end