require 'spec_helper'

module SpAllocationSchedulerTest
  describe 'SpAllocationScheduler node' do
    include_context 'request metadata'
    sp_recommendations_quote = {}

    before(:each) {
      ensure_quote_presence(1, TEST_CUSTOMER[:owner_id])
      variables = {customerInfo: CUSTOMER_INFO}
      get_api_response(variables, :spRecommendations)
      # use the first quote returned from a spRecommendations request for node requests/verifications
      sp_recommendations_quote = json_body[:data][:spRecommendations][:edges][0][:node]
    }

    it 'node route returns expected quote details' do
      wrap_request do
        variables = {}
        graphql url(service: service),
          format_request(
            :node,
            service: service,
            crn: sp_recommendations_quote[:id],
            type: 'SpRecommendation',
            fields: 'id, quoteId, status, accountId, createdAt, finishedAt, customerId, manifestS3path'
          ),
          variables, headers
        expect_success_graphql
        check(:node, service: service)
        node_quote = json_body[:data][:node]
        # verify data consistency between quote returned via spRecommendations,
        # and quote returned via node endpoint
        expect(node_quote[:id]).to eq(sp_recommendations_quote[:id])
        expect(node_quote[:createdAt]).to eq(sp_recommendations_quote[:createdAt])
        expect(node_quote[:accountId]).to eq(sp_recommendations_quote[:accountId])
        expect(node_quote[:quoteId]).to eq(sp_recommendations_quote[:quoteId])
        expect(node_quote[:manifestS3path]).to eq(sp_recommendations_quote[:manifestS3path])
        expect(['CREATED', 'RUNNING', 'COMPLETED', 'FAILED']).to include(node_quote[:status])
       end
    end

    context 'Negative tests: node route' do
      xit 'returns expected error for no auth request' do
        wrap_request do
          variables = {}
          graphql url(service: service),
            format_request(
              :node,
              service: service,
              crn: sp_recommendations_quote[:id],
              type: 'SpRecommendation',
              fields: 'id, quoteId, status, accountId, createdAt, finishedAt, customerId, manifestS3path'
            ),
            variables, apitest_headers
          expect_status(401)
        end
      end

      it 'returns expected error for Invalid CRN' do
        wrap_request do
          variables = {}
          graphql url(service: service),
            format_request(
              :node,
              service: service,
              crn: '1:Recommendation/4567',
              type: 'SpRecommendation',
              fields: 'id, quoteId, status, accountId, createdAt, finishedAt, customerId, manifestS3path'
            ),
            variables, headers
          expect_status(400)
        end
      end

      it 'returns expected error for unsupported type' do
        wrap_request do
          variables = {}
          graphql url(service: service),
            format_request(
              :node,
              service: service,
              crn: sp_recommendations_quote[:id],
              type: 'SpRecommendationInput',
              fields: 'id, quoteId, status, accountId, createdAt, finishedAt, customerId, manifestS3path'
            ),
            variables, headers
          expect_status(400)
        end
      end

      it 'returns expected error for unsupported field' do
        wrap_request do
          variables = {}
          graphql url(service: service),
            format_request(
              :node,
              service: service,
              crn: sp_recommendations_quote[:id],
              type: 'SpRecommendation',
              fields: 'startedAt'
            ),
            variables, headers
          expect_status(400)
        end
      end
    end
  end
end
